package com.xynet.marketing.utils.redis.utils;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 缓存接口
 *
 * @param <V>
 * @author lijin
 */
public interface RedisService<V> {

    /**
     * 设置有效期
     */
    void timeout(String key, long timeout);

    /**
     * 设置有效期
     */
    void timeout(String key, long timeout, TimeUnit timeUnit);

    /**
     * 获取有效期
     *
     * @param key
     * @return
     */
    long getTimeout(String key);

    /**
     * 获取有效期
     *
     * @param key
     * @return
     */
    long getTimeout(String key, TimeUnit timeUnit);

    /**
     * 写入
     */
    void set(String key, V value);

    /**
     * 写入
     */
    void set(String key, V value, long timeout);

    /**
     * 写入
     */
    void set(Map<String, V> map);

    /**
     * 写入
     */
    void set(Map<String, V> map, long timeout);

    /**
     * 是否存在
     */
    boolean exists(String key);

    /**
     * 获取
     */
    V get(String key);

    /**
     * 获取
     */
    Map<String, V> get(List<String> keys);

    /**
     * 模糊获取
     */
    Map<String, V> like(String prex);

    /**
     * 删除
     */
    V remove(String key);

    /**
     * 删除
     */
    Map<String, V> remove(List<String> keys);

    /**
     * 模糊删除
     */
    Map<String, V> removeLike(String prex);

    /**
     * 写入集合
     */
    void setList(String key, V v);

    /**
     * 写入集合
     */
    void setList(String key, List<V> list);

    /**
     * 获取集合最前一个元素并移除
     */
    V getFirstListAndRemove(String key);

    /**
     * 获取集合最后一个元素并移除
     */
    V getLastListAndRemove(String key);

    /**
     * 获取集合
     */
    List<V> getList(String key);

    /**
     * 模糊获取集合
     *
     * @param key
     * @return
     */
    List<V> getLikeList(String key);

    /**
     * 获取集合长度
     */
    int getListSize(String key);

    /**
     * 修改集合
     */
    void updateList(String key, int index, V value);

    /**
     * 删除集合
     */
    V removeList(String key, int index);

    /**
     * 删除集合
     */
    boolean removeList(String key, V v);

    /**
     * 删除集合
     */
    void removeList(String key);

    List<V> removeLikeList(String key);

    /**
     * 写入map
     */
    void setMap(String key, String mk, V mv);

    /**
     * 写入map
     */
    void setMap(String key, Map<String, V> map);

    /**
     * 获取map
     */
    Map<String, V> getMap(String key);

    /**
     * 获取map
     */
    V getMap(String key, String mk);

    /**
     * 模糊查询map
     */
    Map<String, Map<String, V>> likeMap(String prex);

    /**
     * 获取map长度
     *
     * @param key
     * @return
     */
    int getMapSize(String key);

    /**
     * 删除map
     */
    void removeMap(String key);

    /**
     * 删除map
     */
    void removeMap(String key, String mk);

    /**
     * 模糊删除map
     *
     * @param prex
     */
    void removeLikeMap(String prex);

    /**
     * 原子自增
     */
    long atomicIncrease(String key);

    /**
     * 原子自减
     */
    long atomicReduce(String key);

    /**
     * 设置原子值
     */
    long setAtomic(String key, long value);

    /**
     * 获取原子
     */
    long getAtomic(String key);

    /**
     * 原子变量0
     */
    void atomicEmpty(String key);

}
